@@ -1,36 +1,44 @@
 """Constants for commonmeta-py"""
 
-from typing import List, Optional, TypedDict
+from __future__ import annotations
+
+from typing import TypedDict
 
 
-class Commonmeta(TypedDict):
-    """TypedDict for Commonmeta"""
+class CommonmetaRequired(TypedDict):
+    """Required fields for Commonmeta"""
 
     id: str
     type: str
-    url: str
-    creators: List[dict]
-    titles: List[dict]
-    publisher: dict
-    date: dict
-    additional_type: Optional[str]
-    subjects: Optional[List[dict]]
-    contributors: Optional[List[dict]]
-    language: Optional[str]
-    identifiers: Optional[List[dict]]
-    relations: Optional[List[dict]]
-    sizes: Optional[List[dict]]
-    formats: Optional[List[dict]]
-    version: Optional[str]
-    license: Optional[dict]
-    descriptions: Optional[List[dict]]
-    geo_locations: Optional[List[dict]]
-    funding_references: Optional[List[dict]]
-    references: Optional[List[dict]]
-    container: Optional[dict]
-    files: Optional[List[dict]]
-    agency: Optional[str]
-    state: str
+
+
+class Commonmeta(CommonmetaRequired, total=False):
+    """TypedDict for Commonmeta"""
+
+    # optional fields
+    additional_type: str | None
+    agency: str | None
+    container: dict | None
+    contributors: list[dict] | None
+    creators: list[dict] | None
+    date: dict | None
+    descriptions: list[dict] | None
+    files: list[dict] | None
+    formats: list[dict] | None
+    funding_references: list[dict] | None
+    geo_locations: list[dict] | None
+    identifiers: list[dict] | None
+    language: str | None
+    license: dict | None
+    publisher: dict | None
+    references: list[dict] | None
+    relations: list[dict] | None
+    sizes: list[dict] | None
+    state: str | None
+    subjects: list[dict] | None
+    titles: list[dict] | None
+    url: str | None
+    version: str | None
 
 
 # source: https://www.bibtex.com/e/entry-types/
@@ -475,7 +483,7 @@ CM_TO_SO_TRANSLATIONS = {
     "Event": "CreativeWork",
     "Figure": "CreativeWork",
     "Image": "CreativeWork",
-    "Instrument": "Instrument",
+    "Instrument": "Product",
     "JournalArticle": "ScholarlyArticle",
     "LegalDocument": "Legislation",
     "Software": "SoftwareSourceCode",
