@@ -1,12 +1,14 @@
 """Metadata"""
 
+from __future__ import annotations
+
 from os import path
-from typing import Any, Dict, List, Optional, Union
+from typing import Any
 
 import orjson as json
 import yaml
 
-from .base_utils import dig, omit, parse_xml, wrap
+from .base_utils import dig, parse_xml, wrap
 from .file_utils import write_output
 from .readers.cff_reader import get_cff, read_cff
 from .readers.codemeta_reader import (
@@ -70,22 +72,9 @@ from .writers.schema_org_writer import write_schema_org, write_schema_org_list
 class Metadata:
     """Metadata"""
 
-    def __init__(self, string: Optional[Union[str, Dict[str, Any]]], **kwargs):
-        if (
-            string is None
-            or not isinstance(string, (str, dict))
-            and not is_chain_object
-        ):
-            raise ValueError("No input found")
+    def __init__(self, string: str | dict[str, Any] | None, **kwargs):
         self.via = kwargs.get("via", None)
-        if isinstance(string, dict):
-            data = string
-        # if string is an InvenioRDM chain object or any ChainObject-like object
-        elif is_chain_object:
-            data = string._child
-            self.via = "inveniordm"
-            kwargs["parent_doi"] = dig(string._parent.pids, "doi.identifier")
-        elif isinstance(string, str):
+        if isinstance(string, str):
             pid = normalize_id(string)
             if pid is not None and self.via is None:
                 self.via = find_from_format(pid=pid)
@@ -97,6 +86,20 @@ class Metadata:
             if self.via is None:
                 self.via = "commonmeta"
             data = self.get_metadata(pid=pid, string=string)
+        # if string is an InvenioRDM chain object
+        elif is_chain_object(string):
+            self.via = "inveniordm"
+            data = getattr(string, "_child", None)
+            if data is None:
+                raise ValueError("No valid input found")
+            parent = getattr(string, "_parent", None)
+            if parent is not None and hasattr(parent, "pids"):
+                kwargs["parent_doi"] = dig(parent.pids, "doi.identifier")
+        elif isinstance(string, dict):
+            data = string
+        else:
+            raise ValueError("No valid input found")
+
         meta = self.read_metadata(data=data, **kwargs)
 
         # required properties
@@ -171,7 +174,7 @@ class Metadata:
         # Default fallback
         raise ValueError("No metadata found")
 
-    def _get_metadata_from_pid(self, pid, via) -> Dict[str, Any]:
+    def _get_metadata_from_pid(self, pid, via) -> dict[str, Any]:
         """Helper method to get metadata from a PID."""
         if via == "schema_org":
             return get_schema_org(pid)
@@ -194,7 +197,7 @@ class Metadata:
         else:
             return {"pid": pid}
 
-    def _get_metadata_from_string(self, string, via) -> Dict[str, Any]:
+    def _get_metadata_from_string(self, string, via) -> dict[str, Any]:
         """Helper method to get metadata from a string."""
         try:
             # XML formats
@@ -238,7 +241,7 @@ class Metadata:
         except (TypeError, json.JSONDecodeError) as error:
             return {"error": str(error)}
 
-    def read_metadata(self, data: Dict[str, Any], **kwargs) -> Dict[str, Any]:
+    def read_metadata(self, data: dict[str, Any], **kwargs) -> dict[str, Any]:
         """Read and parse metadata from various formats."""
         via = (isinstance(data, dict) and data.get("via")) or self.via
 
@@ -275,33 +278,37 @@ class Metadata:
         else:
             raise ValueError("No input format found")
 
-    def write(self, to: str = "commonmeta", **kwargs) -> Union[str, bytes]:
+    def write(self, to: str = "commonmeta", **kwargs) -> bytes | None:
         """convert metadata list into different formats"""
         try:
-            result = self._write_format(to, **kwargs)
-            if result is None or result == "":
-                return "{}"
-            return result
+            return self._write_format(to, **kwargs)
         except json.JSONDecodeError as e:
             # More specific error message including the original JSONDecodeError details
             raise ValueError(f"Invalid JSON: {str(e)}")
+        except Exception as e:
+            raise ValueError(f"Error writing format '{to}': {str(e)}")
 
-    def _write_format(self, to: str, **kwargs) -> Union[str, bytes]:
+    def _write_format(self, to: str, **kwargs) -> bytes | None:
         """Helper method to handle writing to different formats."""
         # JSON-based output formats
-        if to == "commonmeta":
-            result = json.dumps(write_commonmeta(self))
-        elif to == "datacite":
-            result = json.dumps(write_datacite(self))
-        elif to == "inveniordm":
-            result = json.dumps(write_inveniordm(self))
-        elif to == "schema_org":
-            result = json.dumps(write_schema_org(self))
+        if to in ["commonmeta", "datacite", "inveniordm", "schema_org", "csl"]:
+            writer_map = {
+                "commonmeta": write_commonmeta,
+                "datacite": write_datacite,
+                "inveniordm": write_inveniordm,
+                "schema_org": write_schema_org,
+                "csl": write_csl,
+            }
+            output = writer_map[to](self)
+            self.write_errors = json_schema_errors(output, schema=to)
+            if self.write_errors is not None:
+                self.is_valid = False
+            if output is None:
+                return b"{}"
+            return json.dumps(output)
         # Text-based output formats
         elif to == "bibtex":
             return write_bibtex(self)
-        elif to == "csl":
-            return self._write_csl(**kwargs)
         elif to == "citation":
             self.style = kwargs.get("style", "apa")
             self.locale = kwargs.get("locale", "en-US")
@@ -310,65 +317,18 @@ class Metadata:
             return write_ris(self)
         # XML-based output formats
         elif to == "crossref_xml":
-            return self._write_crossref_xml(**kwargs)
+            self.depositor = kwargs.get("depositor", None)
+            self.email = kwargs.get("email", None)
+            self.registrant = kwargs.get("registrant", None)
+            output = write_crossref_xml(self)
+            self.write_errors = xml_schema_errors(output, schema="crossref_xml")
+            if self.write_errors is not None:
+                self.is_valid = False
+            return output
         else:
-            raise ValueError("No output format found")
+            raise ValueError(f"Unsupported output format: {to}")
 
-        if isinstance(result, str):
-            # Verify it's valid JSON
-            try:
-                json.loads(result)
-                return result
-            except json.JSONDecodeError:
-                return "{}"
-        elif result is not None:
-            try:
-                decoded = result.decode("utf-8")
-                # Verify it's valid JSON
-                json.loads(decoded)
-                return decoded
-            except (json.JSONDecodeError, UnicodeDecodeError):
-                return "{}"
-        return "{}"
-
-    def _write_csl(self, **kwargs) -> str:
-        """Write in CSL format with error checking."""
-        csl_output = write_csl(self)
-        if csl_output:
-            instance = omit(json.loads(csl_output), [])
-            self.errors = json_schema_errors(instance, schema="csl")
-            return csl_output
-        return ""
-
-    def _write_datacite(self) -> str:
-        """Write in DataCite format with error checking."""
-        datacite_output = write_datacite(self)
-        if not datacite_output:
-            return ""
-        try:
-            instance = json.loads(datacite_output)
-            self.write_errors = json_schema_errors(instance, schema="datacite")
-            return str(datacite_output)
-        except (json.JSONDecodeError, TypeError):
-            return "{}" if not datacite_output else str(datacite_output)
-
-    def _write_crossref_xml(self, **kwargs) -> str:
-        """Write in Crossref XML format with error checking."""
-        # doi = doi_from_url(self.id)
-        # _type = CM_TO_CR_TRANSLATIONS.get(str(self.type or ""), None)
-        # url = self.url
-        # instance = {"doi": doi, "type": _type, "url": url}
-        self.depositor = kwargs.get("depositor", None)
-        self.email = kwargs.get("email", None)
-        self.registrant = kwargs.get("registrant", None)
-        output = write_crossref_xml(self)
-        self.write_errors = xml_schema_errors(output, schema="crossref_xml")
-        if self.write_errors is not None:
-            self.is_valid = False
-            return ""
-        return output if output is not None else ""
-
-    def push(self, to: str = "commonmeta", **kwargs) -> Union[str, bytes]:
+    def push(self, to: str = "commonmeta", **kwargs) -> str | dict:
         """push metadata to external APIs"""
 
         if to == "crossref_xml":
@@ -396,7 +356,7 @@ class MetadataList:
     """MetadataList"""
 
     def __init__(
-        self, dct: Optional[Union[str, Dict[str, Any]]] = None, **kwargs
+        self, dct: str | bytes | dict[str, Any] | None = None, **kwargs
     ) -> None:
         if dct is None or not isinstance(dct, (str, bytes, dict)):
             raise ValueError("No input found")
@@ -438,7 +398,7 @@ class MetadataList:
         # other options
         self.file = kwargs.get("file", None)
 
-    def get_metadata_list(self, string) -> list:
+    def get_metadata_list(self, string) -> dict:
         if string is None or not isinstance(string, (str, bytes)):
             raise ValueError("No input found")
         if self.via in [
@@ -458,21 +418,25 @@ class MetadataList:
         else:
             raise ValueError("No input format found")
 
-    def read_metadata_list(self, items, **kwargs) -> List[Metadata]:
+    def read_metadata_list(self, items, **kwargs) -> list[Metadata]:
         """read_metadata_list"""
         kwargs["via"] = kwargs.get("via", None) or self.via
         return [Metadata(i, **kwargs) for i in items]
 
-    def write(self, to: str = "commonmeta", **kwargs) -> Union[str, bytes]:
+    def write(self, to: str = "commonmeta", **kwargs) -> str | bytes | None:
         """convert metadata list into different formats"""
         if to == "bibtex":
             output = write_bibtex_list(self)
-            if self.file:
+            if self.file and output is not None:
                 return write_output(self.file, output, [".bib"])
             else:
                 return output
         elif to == "citation":
-            return write_citation_list(self, **kwargs)
+            output = write_citation_list(self, **kwargs)
+            if self.file and output is not None:
+                return write_output(self.file, output, [".html"])
+            else:
+                return output
         elif to == "commonmeta":
             output = json.dumps(write_commonmeta_list(self))
             if self.file:
@@ -481,7 +445,7 @@ class MetadataList:
                 return output
         elif to == "crossref_xml":
             output = write_crossref_xml_list(self)
-            if self.file:
+            if self.file and output is not None:
                 return write_output(self.file, output, [".xml"])
             else:
                 return output
@@ -504,7 +468,11 @@ class MetadataList:
             else:
                 return output
         elif to == "ris":
-            return write_ris_list(self)
+            output = write_ris_list(self)
+            if self.file and output is not None:
+                return write_output(self.file, output, [".ris"])
+            else:
+                return output
         elif to == "schema_org":
             output = json.dumps(write_schema_org_list(self))
             if self.file:
@@ -514,7 +482,7 @@ class MetadataList:
         else:
             raise ValueError("No valid output format found")
 
-    def push(self, to: str = "commonmeta", **kwargs) -> Union[str, bytes]:
+    def push(self, to: str = "commonmeta", **kwargs) -> bytes | None:
         """push metadata list to external APIs"""
 
         if to == "crossref_xml":
