@@ -1,18 +1,25 @@
 """Bibtex writer for commonmeta-py"""
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from bibtexparser.bibdatabase import BibDatabase
 from bibtexparser.bwriter import BibTexWriter
 from bibtexparser.customization import page_double_hyphen
 
 from ..author_utils import authors_as_string
 from ..base_utils import compact
-from ..constants import CM_TO_BIB_TRANSLATIONS, Commonmeta
+from ..constants import CM_TO_BIB_TRANSLATIONS
 from ..date_utils import MONTH_SHORT_NAMES, get_iso8601_date, get_month_from_date
 from ..doi_utils import doi_from_url
 from ..utils import get_language, pages_as_string
 
+if TYPE_CHECKING:
+    from ..metadata import Metadata, MetadataList
 
-def write_bibtex(metadata: Commonmeta) -> str:
+
+def write_bibtex(metadata: Metadata) -> bytes | None:
     """Write bibtex"""
     if metadata.write_errors is not None:
         return None
@@ -33,10 +40,10 @@ def write_bibtex(metadata: Commonmeta) -> str:
     # Hack to remove curly braces around month names
     for month_name in MONTH_SHORT_NAMES:
         bibtex_str = bibtex_str.replace(f"{{{month_name}}}", month_name)
-    return bibtex_str
+    return bibtex_str.encode("utf-8")
 
 
-def write_bibtex_item(metadata: Commonmeta) -> dict:
+def write_bibtex_item(metadata: Metadata) -> dict:
     """Write bibtex item"""
     container = metadata.container if metadata.container else {}
     date_published = get_iso8601_date(metadata.date.get("published", None))
@@ -62,7 +69,11 @@ def write_bibtex_item(metadata: Commonmeta) -> dict:
     )
     author = authors if authors and len(authors) > 0 else None
     license_ = str(metadata.license.get("url")) if metadata.license else None
-    institution = metadata.publisher.get("name", None) if _type == "phdthesis" else None
+    institution = (
+        metadata.publisher.get("name", None)
+        if _type == "phdthesis" and metadata.publisher
+        else None
+    )
     issn = (
         container.get("identifier", None)
         if container.get("identifierType", None) == "ISSN"
@@ -93,7 +104,7 @@ def write_bibtex_item(metadata: Commonmeta) -> dict:
     pages = pages_as_string(container)
     publisher = (
         metadata.publisher.get("name", None)
-        if _type not in ["article", "phdthesis"]
+        if _type not in ["article", "phdthesis"] and metadata.publisher
         else None
     )
     series = container.get("series", None)
@@ -128,7 +139,7 @@ def write_bibtex_item(metadata: Commonmeta) -> dict:
     )
 
 
-def write_bibtex_list(metalist):
+def write_bibtex_list(metalist: MetadataList | None) -> bytes | None:
     """Write bibtex list"""
     if metalist is None:
         return None
@@ -147,4 +158,4 @@ def write_bibtex_list(metalist):
     # TODO: Fix this in write_bibtex_item
     for month_name in MONTH_SHORT_NAMES:
         bibtex_str = bibtex_str.replace(f"{{{month_name}}}", month_name)
-    return bibtex_str
+    return bibtex_str.encode("utf-8")
