@@ -1,10 +1,12 @@
 """Crossref XML writer for commonmeta-py"""
 
+from __future__ import annotations
+
 import io
 import logging
 from datetime import datetime
 from time import time
-from typing import Dict, Optional, Union
+from typing import TYPE_CHECKING, Dict
 
 import orjson as json
 import requests
@@ -12,14 +14,24 @@ from dateutil.parser import parse as date_parse
 from furl import furl
 from isbnlib import canonical, is_isbn10, is_isbn13
 from marshmallow import Schema, fields
+from requests.exceptions import RequestException
 from requests_toolbelt.multipart.encoder import MultipartEncoder
 
-from ..base_utils import compact, dig, parse_xml, unparse_xml, unparse_xml_list, wrap
-from ..constants import Commonmeta
+from ..base_utils import (
+    compact,
+    dig,
+    parse_xml,
+    unparse_xml,
+    unparse_xml_list,
+    wrap,
+)
 from ..doi_utils import doi_from_url, is_rogue_scholar_doi, validate_doi
 from ..utils import validate_url
 from .inveniordm_writer import push_inveniordm, update_legacy_record
 
+if TYPE_CHECKING:
+    from ..metadata import Metadata, MetadataList
+
 log = logging.getLogger(__name__)
 
 POSTED_CONTENT_TYPES = [
@@ -86,11 +98,12 @@ class CrossrefXMLSchema(Schema):
     license = fields.Dict(data_key="ai:program")
     relations = fields.Dict(data_key="rel:program")
     archive_locations = fields.List(fields.Dict())
+    version_info = fields.Dict()
     doi_data = fields.Dict(data_key="doi_data")
     references = fields.Dict(data_key="citation_list")
 
 
-def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
+def convert_crossref_xml(metadata: Metadata) -> dict | None:
     """Convert Crossref XML"""
 
     # return None if type is not supported by Crossref
@@ -145,6 +158,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "funding_references": funding_references,
                 "license": license,
                 "relations": relations,
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
             }
@@ -165,6 +179,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "funding_references": funding_references,
                 "license": license,
                 "relations": relations,
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
             }
@@ -206,6 +221,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "license": license,
                 "relations": relations,
                 "archive_locations": get_archive_locations(metadata),
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
             }
@@ -235,6 +251,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "titles": titles,
                 "contributors": contributors,
                 "publication_date": get_publication_date(metadata, media_type="online"),
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
             }
         )
@@ -252,6 +269,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "funding_references": funding_references,
                 "license": license,
                 "relations": relations,
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
             }
         )
@@ -275,6 +293,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "crossmark": None,
                 "relations": relations,
                 "archive_locations": get_archive_locations(metadata),
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
             }
@@ -315,6 +334,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "crossmark": None,
                 "relations": relations,
                 "archive_locations": get_archive_locations(metadata),
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
             }
@@ -335,6 +355,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
                 "crossmark": None,
                 "relations": relations,
                 "archive_locations": get_archive_locations(metadata),
+                "version_info": get_version_info(metadata),
                 "doi_data": doi_data,
                 "references": references,
                 "component_list": None,
@@ -346,7 +367,7 @@ def convert_crossref_xml(metadata: Commonmeta) -> Optional[dict]:
     return data
 
 
-def write_crossref_xml(metadata: Commonmeta) -> Optional[str]:
+def write_crossref_xml(metadata: Metadata) -> bytes | None:
     """Write Crossref XML"""
     if metadata is None or not metadata.is_valid:
         log.error("Invalid metadata provided for Crossref XML generation")
@@ -376,7 +397,7 @@ def write_crossref_xml(metadata: Commonmeta) -> Optional[str]:
     return unparse_xml(crossref_xml, dialect="crossref", head=head)
 
 
-def write_crossref_xml_list(metalist) -> Optional[str]:
+def write_crossref_xml_list(metalist: MetadataList) -> bytes | None:
     """Write crossref_xml list"""
     if metalist is None or not metalist.is_valid:
         log.error("Invalid metalist provided for Crossref XML generation")
@@ -406,7 +427,7 @@ def write_crossref_xml_list(metalist) -> Optional[str]:
 
 
 def push_crossref_xml(
-    metadata: Commonmeta,
+    metadata: Metadata,
     login_id: str,
     login_passwd: str,
     test_mode: bool,
@@ -463,20 +484,20 @@ def push_crossref_xml(
 
 
 def push_crossref_xml_list(
-    metalist,
+    metalist: MetadataList,
     login_id: str,
     login_passwd: str,
     test_mode: bool,
     host: str,
     token: str,
     legacy_key: str,
-) -> str:
+) -> bytes | None:
     """Push crossref_xml list to Crossref API, returns the API response."""
 
     input_xml = write_crossref_xml_list(metalist)
     if not input_xml:
         log.error("Failed to generate XML for upload")
-        return "{}"
+        return None
 
     client = CrossrefXMLClient(
         username=login_id,
@@ -487,7 +508,7 @@ def push_crossref_xml_list(
 
     if status != "SUCCESS":
         log.error("Failed to upload XML to Crossref")
-        return "{}"
+        return None
 
     items = []
     for item in metalist.items:
@@ -521,7 +542,7 @@ def push_crossref_xml_list(
         items.append(record)
 
     # Return JSON response
-    return json.dumps(items, option=json.OPT_INDENT_2).decode("utf-8")
+    return json.dumps(items, option=json.OPT_INDENT_2)
 
 
 def get_attributes(obj, **kwargs) -> dict:
@@ -537,7 +558,7 @@ def get_attributes(obj, **kwargs) -> dict:
     )
 
 
-def get_journal_metadata(obj) -> Optional[dict]:
+def get_journal_metadata(obj) -> dict | None:
     """get journal metadata"""
     issn = (
         dig(obj, "container.identifier")
@@ -553,7 +574,7 @@ def get_journal_metadata(obj) -> Optional[dict]:
     )
 
 
-def get_book_metadata(obj) -> Optional[dict]:
+def get_book_metadata(obj) -> dict | None:
     return compact(
         {
             "@language": dig(obj, "language"),
@@ -561,7 +582,7 @@ def get_book_metadata(obj) -> Optional[dict]:
     )
 
 
-def get_database_metadata(obj) -> Optional[dict]:
+def get_database_metadata(obj) -> dict | None:
     return compact(
         {
             "@language": dig(obj, "language"),
@@ -569,7 +590,7 @@ def get_database_metadata(obj) -> Optional[dict]:
     )
 
 
-def get_event_metadata(obj) -> Optional[dict]:
+def get_event_metadata(obj) -> dict | None:
     """get event metadata"""
     if dig(obj, "container.title") is None:
         return None
@@ -583,7 +604,7 @@ def get_event_metadata(obj) -> Optional[dict]:
     )
 
 
-def get_proceedings_metadata(obj) -> Optional[dict]:
+def get_proceedings_metadata(obj) -> dict | None:
     """get proceedings metadata"""
     if dig(obj, "container.title") is None or dig(obj, "publisher.name") is None:
         return None
@@ -601,7 +622,7 @@ def get_proceedings_metadata(obj) -> Optional[dict]:
     return compact(proceedings_metadata)
 
 
-def get_journal_issue(obj) -> Optional[dict]:
+def get_journal_issue(obj) -> dict | None:
     """get journal issue"""
     volume = dig(obj, "container.volume")
     if volume is not None:
@@ -615,7 +636,7 @@ def get_journal_issue(obj) -> Optional[dict]:
     )
 
 
-def get_institution(obj) -> Optional[dict]:
+def get_institution(obj) -> dict | None:
     """get institution"""
     if dig(obj, "publisher.name") is None:
         return None
@@ -625,7 +646,7 @@ def get_institution(obj) -> Optional[dict]:
     }
 
 
-def get_titles(obj) -> Optional[dict]:
+def get_titles(obj) -> dict | None:
     """get titles"""
 
     titles = {}
@@ -640,7 +661,7 @@ def get_titles(obj) -> Optional[dict]:
     return titles if titles else None
 
 
-def get_contributors(obj) -> Optional[dict]:
+def get_contributors(obj) -> dict | None:
     """get contributors"""
 
     def map_affiliations(affiliations):
@@ -742,7 +763,7 @@ def get_contributors(obj) -> Optional[dict]:
     return result if result else None
 
 
-def get_publisher(obj) -> Optional[dict]:
+def get_publisher(obj) -> dict | None:
     """get publisher"""
     if dig(obj, "publisher.name") is None:
         return None
@@ -752,7 +773,7 @@ def get_publisher(obj) -> Optional[dict]:
     }
 
 
-def get_abstracts(obj) -> Optional[list]:
+def get_abstracts(obj) -> list | None:
     """get abstracts"""
     if len(wrap(dig(obj, "descriptions"))) == 0:
         return None
@@ -776,12 +797,12 @@ def get_abstracts(obj) -> Optional[list]:
     return abstracts
 
 
-def get_group_title(obj) -> Optional[str]:
+def get_group_title(obj) -> str | None:
     """Get group title from metadata"""
     return dig(obj, "container.title")
 
 
-def get_item_number(obj) -> Optional[dict]:
+def get_item_number(obj) -> dict | None:
     """Insert item number"""
     if len(wrap(dig(obj, "identifiers"))) == 0:
         return None
@@ -795,7 +816,7 @@ def get_item_number(obj) -> Optional[dict]:
             }
 
 
-def get_publication_date(obj, media_type: Optional[str] = None) -> Optional[Dict]:
+def get_publication_date(obj, media_type: str | None = None) -> Dict | None:
     """get publication date"""
     pub_date_str = dig(obj, "date.published")
     if pub_date_str is None:
@@ -817,7 +838,7 @@ def get_publication_date(obj, media_type: Optional[str] = None) -> Optional[Dict
     )
 
 
-def get_archive_locations(obj) -> Optional[list]:
+def get_archive_locations(obj) -> list | None:
     """get archive locations"""
     if len(wrap(dig(obj, "archive_locations"))) == 0:
         return None
@@ -832,7 +853,15 @@ def get_archive_locations(obj) -> Optional[list]:
     ]
 
 
-def get_references(obj) -> Optional[Dict]:
+def get_version_info(obj) -> dict | None:
+    """get version_info"""
+    if dig(obj, "version") is None:
+        return None
+    # TODO: add optional description and xml:lang attributes
+    return compact({"version": dig(obj, "version")})
+
+
+def get_references(obj) -> Dict | None:
     """get references"""
     if len(wrap(dig(obj, "references"))) == 0:
         return None
@@ -869,7 +898,7 @@ def get_references(obj) -> Optional[Dict]:
     return {"citation": citations}
 
 
-def get_license(obj) -> Optional[dict]:
+def get_license(obj) -> dict | None:
     """get license"""
     rights_uri = dig(obj, "license.url")
     if rights_uri is None:
@@ -891,7 +920,7 @@ def get_license(obj) -> Optional[dict]:
     }
 
 
-def get_funding_references(obj) -> Optional[dict]:
+def get_funding_references(obj) -> dict | None:
     """Get funding references"""
     if len(wrap(dig(obj, "funding_references"))) == 0:
         return None
@@ -976,7 +1005,7 @@ def get_funding_references(obj) -> Optional[dict]:
     }
 
 
-def get_relations(obj) -> Optional[Dict]:
+def get_relations(obj) -> Dict | None:
     """get relations"""
     if len(wrap(dig(obj, "relations"))) == 0:
         return None
@@ -1053,7 +1082,7 @@ def get_relations(obj) -> Optional[Dict]:
     }
 
 
-def get_subjects(obj) -> Optional[list]:
+def get_subjects(obj) -> list | None:
     """Get crossref subjects"""
     if dig(obj, "subjects") is None:
         return None
@@ -1066,7 +1095,7 @@ def get_subjects(obj) -> Optional[list]:
     return subjects
 
 
-def get_doi_data(obj) -> Optional[dict]:
+def get_doi_data(obj) -> dict | None:
     """get doi data"""
     if doi_from_url(dig(obj, "id")) is None or dig(obj, "url") is None:
         return None
@@ -1102,7 +1131,7 @@ def get_doi_data(obj) -> Optional[dict]:
     )
 
 
-def get_isbn(obj, media_type: Optional[str] = None) -> Optional[Dict]:
+def get_isbn(obj, media_type: str | None = None) -> str | None:
     """get isbn"""
     if (
         dig(obj, "container.identifierType") != "ISBN"
@@ -1113,7 +1142,7 @@ def get_isbn(obj, media_type: Optional[str] = None) -> Optional[Dict]:
     return normalize_isbn_crossref(isbn)
 
 
-def normalize_isbn_crossref(isbn: str) -> Optional[str]:
+def normalize_isbn_crossref(isbn: str) -> str | None:
     """Normalize ISBN for Crossref XML.
 
     Crossref XSD pattern: (97(8|9)-)?\\d[\\d \\-]+[\\dX]
@@ -1174,7 +1203,7 @@ class CrossrefXMLClient:
         else:
             self.api_url = "https://doi.crossref.org/servlet/deposit"
 
-    def post(self, input_xml: Union[str, bytes]) -> str:
+    def post(self, input_xml: str | bytes) -> str:
         """Upload metadata for a new or existing DOI.
 
         :param input_xml: XML metadata following the Crossref Metadata Schema (str or bytes).
@@ -1207,7 +1236,7 @@ class CrossrefXMLClient:
 
             return self._parse_response(resp)
 
-        except requests.exceptions.RequestException as e:
+        except RequestException as e:
             log.error(f"Failed to upload to Crossref: {e}")
             return "{}"
 
